/******************************************************************************/
/*                Copyright (c) 2016 MD Automotive Controls.                  */
/*                         All rights reserved.                               */
/******************************************************************************/
/*                                                                            */
/* PACKAGE TITLE:      LSU4.X Header File                                     */
/* DESCRIPTION:        This code module initialises the required ADC, DAC     */
/*                     CTRL, spread, table, DIO and TEPM resources for        */
/*                     managing the nernst cell voltage and impedence         */
/*                     measurement for the LSU4.X sensor/s                    */
/* FILE NAME:          LSU4X.h                                                */
/* REVISION HISTORY:   07-03-2016 | 1.0 | Initial revision                    */
/*                                                                            */
/******************************************************************************/
#ifndef LSU4X_H
#define LSU4X_H

#include "ADCAPI.h"
#include "DACAPI.h"
#include "CTRLAPI.h"
#include "CONV.h"
#include "declarations.h"
#include "IOAPI.h"
#include "LSUH.h"
#include "SPREADSAPI.h"
#include "TABLESAPI.h"
#include "TEPMAPI.h"
#include "UNITS.h"
#include "USER.h"
#include "USERMATH.h"


/* GLOBAL MACRO DEFINITIONS ***************************************************/
#define LSU4X_nDACQueueSize 		1u
#define LSU4X_nDACRange             3200u
#define LSU4X_nADCRange             4095u
#define LSU4X_nPWMCycCount          40u
#define LSU4X_nStartACSample		5u
#define LSU4X_nEndACSample          18u
#define LSU4X_nPumpCurrSamples      10u
#define LSU4X_nACSamplesCount       20u
#define LSU4X_nDCSamplesCount       20u
#define LSU4X_nWarmMsMin            4000u
#define LSU4X_nWarmMsMax            10000u
#define LSU4X_nWarmPrePumpMs		2000u
/*CR1_105*/

#define LSU4X_nSWHResource          EH_IO_GP6
#define LSU4X_nSWLResource          EH_IO_GP7
#define LSU4X_nIENResource          EH_IO_GPSE8
#define LSU4X_nDACResource          EH_O_DAC1

#define LSU4X_nADConfig   \
{      \
	{EH_I_ADD1,         IOAPI_enADD,  ADCAPI_en8Samples, ADCAPI_enDiffGain32, &LSU4X_vADCCallBack, ADCAPI_enTrigger1},	\
	{EH_IO_TMR18, 		IOAPI_enADSE, ADCAPI_en4Samples, ADCAPI_enDiffGain8, &LSU4X_vADCCallBack, ADCAPI_enTrigger1},	\
	{EH_IO_TMR17, 		IOAPI_enADSE, ADCAPI_en4Samples, ADCAPI_enDiffGain8, &LSU4X_vADCCallBack, ADCAPI_enTrigger1},	\
	{EH_IO_IIC1_SCL,    IOAPI_enADSE, ADCAPI_en4Samples, ADCAPI_enDiffGain1, &LSU4X_vADCCallBack, ADCAPI_enTrigger1},	\
}


#ifdef EXTERN
	#undef EXTERN
#endif
#ifdef _LSU4X_C
	#define EXTERN
#else
	#define EXTERN extern
#endif


/* GLOBAL TYPE DEFINITIONS ****************************************************/
/*******************************************************************************
* Description      : Type to hold an AD resource configuration
*******************************************************************************/
typedef struct
{
	IOAPI_tenEHIOResource enEHIOResource;
	IOAPI_tenEHIOType enEHIOType;
	ADCAPI_tenSamplesAv enSamplesAv;
	ADCAPI_tenDiffGain enDiffGain;
	ADCAPI_tpfResultCB pfResultCB;	
	ADCAPI_tenTrigger enTrigger;
} LSU4X_tstADConfig;


/* GLOBAL VARIABLE DECLARATIONS ***********************************************/
EXTERN uint32 LSU4X_au32ACSampleCount[LSU_DEVICE_COUNT];
EXTERN uint32 LSU4X_au32DCSampleCount[LSU_DEVICE_COUNT];
EXTERN uint16 LSU4X_u16PWMPeriod;
//REMOVED!!ASAMREMOVED!! mode=readvalue name="PWMPeriod" type=uint8 offset=0 min=0 max=5 m=0.001 b=1 units="dl" format=6.0 help="PWM Period"
EXTERN uint16 LSU4X_u16PWMLastStart;
//REMOVED!!ASAMREMOVED!! mode=readvalue name="PWMLastStart" type=uint8 offset=0 min=0 max=5 m=0.001 b=1 units="dl" format=6.0 help="PWM Period Last Origin"
EXTERN GPM6_ttOhms LSU4X_atNernstSenseOhms[LSU_DEVICE_COUNT];
//REMOVED!!ASAMREMOVED!! mode=readvalue name="Nernst Rout 1" type=uint32 offset=0 min=0 max=1000 m=0.001 b=0 units="Ohm" format=3.0 help="Nernst 1 output impedence"
//REMOVED!!ASAMREMOVED!! mode=readvalue name="Nernst Rout 2" type=uint32 offset=4 min=0 max=1000 m=0.001 b=0 units="Ohm" format=3.0 help="Nernst 2 output impedence"
EXTERN GPM6_ttLambda LSU4X_atLambdaRaw[LSU_DEVICE_COUNT];
//REMOVED!!ASAMREMOVED!! mode=readvalue name="Lambda 1" type=uint16 offset=0 min=0 max=5 m=0.001 b=0 units="dl" format=4.3 help="Lambda 1 Measurement (dimensionless)"
//REMOVED!!ASAMREMOVED!! mode=readvalue name="AFR 1" type=uint16 offset=0 min=0 max=50 m=0.0147 b=0 units="ARF" format=2.3 help="AFR 1 Measurement (dimensionless)"
//REMOVED!!ASAMREMOVED!! mode=readvalue name="Lambda 2" type=uint16 offset=2 min=0 max=5 m=0.001 b=0 units="dl" format=4.3 help="Lambda 2 Measurement (dimensionless)"
EXTERN uint32 LSU4X_aau32ACSamples[LSU_DEVICE_COUNT][LSU4X_nACSamplesCount];
//REMOVED!!ASAMREMOVED!! mode=readvalue name="Samples1" type=uint32 offset=0 min=0 max=5 m=1 b=0 units="n" format=6.0 help="sample 1"
//REMOVED!!ASAMREMOVED!! mode=readvalue name="Samples2" type=uint32 offset=4 min=0 max=5 m=1 b=0 units="n" format=6.0 help="sample 2"
//REMOVED!!ASAMREMOVED!! mode=readvalue name="Samples3" type=uint32 offset=8 min=0 max=5 m=1 b=0 units="n" format=6.0 help="sample 3"
//REMOVED!!ASAMREMOVED!! mode=readvalue name="Samples4" type=uint32 offset=12 min=0 max=5 m=1 b=0 units="n" format=6.0 help="sample 4"
//REMOVED!!ASAMREMOVED!! mode=readvalue name="Samples5" type=uint32 offset=16 min=0 max=5 m=1 b=0 units="n" format=6.0 help="sample 5"
//REMOVED!!ASAMREMOVED!! mode=readvalue name="Samples6" type=uint32 offset=20 min=0 max=5 m=1 b=0 units="n" format=6.0 help="sample 6"
//REMOVED!!ASAMREMOVED!! mode=readvalue name="Samples7" type=uint32 offset=24 min=0 max=5 m=1 b=0 units="n" format=6.0 help="sample 7"
//REMOVED!!ASAMREMOVED!! mode=readvalue name="Samples8" type=uint32 offset=28 min=0 max=5 m=1 b=0 units="n" format=6.0 help="sample 8"
EXTERN sint32 LSU4X_aas32DCSamples[LSU_DEVICE_COUNT][LSU4X_nDCSamplesCount];
EXTERN uint8 LSU4X_au8ACCallBackWait[LSU_DEVICE_COUNT];


/* GLOBAL FUNCTION DECLARATIONS ***********************************************/
void LSU4X_vStart(uint32 * const pu32Arg);
void LSU4X_vRun(uint32* const pu32Arg);
void LSU4X_vTerminate(uint32* const pu32Arg);
void LSU4X_vCallBack(uint32* const pu32Arg);
void LSU4X_vThread(void);

#endif // LSU4X_H

