/******************************************************************************/
/*    Copyright (c) 2016 MD Automotive Controls. Original Work.               */
/*    License: http://www.gnu.org/licenses/gpl.html GPL version 2 or higher   */
/******************************************************************************/
/* CONTEXT:USER_APP                                                           */
/* PACKAGE TITLE:      User Math Header File                                  */
/* DESCRIPTION:        This code module provides mathematical data filtering  */
/*                     functions for the user application                     */
/* FILE NAME:          USERMATH.h                                             */
/* REVISION HISTORY:   07-03-2016 | 1.0 | Initial revision                    */
/*                                                                            */
/*                                                                            */
/*                                                                            */
/*                                                                            */
/******************************************************************************/
#ifndef USERMATH_H
#define USERMATH_H

#include <limits.h>
#include <stddef.h>
#include <stdint.h>
#include <string.h>
#include "declarations.h"
#include "sys.h"
#include "types.h"


/* GLOBAL MACRO DEFINITIONS ***************************************************/
#define ABS(a)   		(((a) > 0) 	? (a) : (-a))

#define USERMATH_nTimeConstantData \
{\
0x1,\
0x1,\
0x1,\
0x1,\
0x1,\
0x1,\
0x1,\
0x1,\
0xB,\
0x1F,\
0x32,\
0x43,\
0x52,\
0x60,\
0x6D,\
0x7A,\
0x82,\
0x88,\
0x8D,\
0x92,\
0x96,\
0x9B,\
0x9F,\
0xA3,\
0xA7,\
0xAB,\
0xAE,\
0xB2,\
0xB5,\
0xB8,\
0xBB,\
0xBE,\
0xC0,\
0xC2,\
0xC3,\
0xC4,\
0xC6,\
0xC7,\
0xC8,\
0xC9,\
0xCA,\
0xCC,\
0xCD,\
0xCE,\
0xCF,\
0xD0,\
0xD1,\
0xD2,\
0xD3,\
0xD4,\
0xD5,\
0xD5,\
0xD6,\
0xD7,\
0xD8,\
0xD9,\
0xDA,\
0xDB,\
0xDB,\
0xDC,\
0xDD,\
0xDE,\
0xDE,\
0xDF,\
0xE0,\
0xE0,\
0xE0,\
0xE1,\
0xE1,\
0xE1,\
0xE2,\
0xE2,\
0xE2,\
0xE3,\
0xE3,\
0xE3,\
0xE4,\
0xE4,\
0xE4,\
0xE5,\
0xE5,\
0xE5,\
0xE5,\
0xE6,\
0xE6,\
0xE6,\
0xE6,\
0xE7,\
0xE7,\
0xE7,\
0xE7,\
0xE8,\
0xE8,\
0xE8,\
0xE8,\
0xE9,\
0xE9,\
0xE9,\
0xE9,\
0xEA,\
0xEA,\
0xEA,\
0xEA,\
0xEB,\
0xEB,\
0xEB,\
0xEB,\
0xEB,\
0xEC,\
0xEC,\
0xEC,\
0xEC,\
0xED,\
0xED,\
0xED,\
0xED,\
0xED,\
0xEE,\
0xEE,\
0xEE,\
0xEE,\
0xEE,\
0xEE,\
0xEF,\
0xEF,\
0xEF,\
0xEF,\
0xEF,\
0xF0,\
0xF0,\
0xF0,\
0xF0,\
0xF0,\
0xF0,\
0xF0,\
0xF0,\
0xF0,\
0xF0,\
0xF0,\
0xF0,\
0xF1,\
0xF1,\
0xF1,\
0xF1,\
0xF1,\
0xF1,\
0xF1,\
0xF1,\
0xF1,\
0xF1,\
0xF1,\
0xF1,\
0xF2,\
0xF2,\
0xF2,\
0xF2,\
0xF2,\
0xF2,\
0xF2,\
0xF2,\
0xF2,\
0xF2,\
0xF2,\
0xF2,\
0xF2,\
0xF2,\
0xF3,\
0xF3,\
0xF3,\
0xF3,\
0xF3,\
0xF3,\
0xF3,\
0xF3,\
0xF3,\
0xF3,\
0xF3,\
0xF3,\
0xF3,\
0xF3,\
0xF3,\
0xF4,\
0xF4,\
0xF4,\
0xF4,\
0xF4,\
0xF4,\
0xF4,\
0xF4,\
0xF4,\
0xF4,\
0xF4,\
0xF4,\
0xF4,\
0xF4,\
0xF4,\
0xF4,\
0xF5,\
0xF5,\
0xF5,\
0xF5,\
0xF5,\
0xF5,\
0xF5,\
0xF5,\
0xF5,\
0xF5,\
0xF5,\
0xF5,\
0xF5,\
0xF5,\
0xF5,\
0xF5,\
0xF5,\
0xF5,\
0xF6,\
0xF6,\
0xF6,\
0xF6,\
0xF6,\
0xF6,\
0xF6,\
0xF6,\
0xF6,\
0xF6,\
0xF6,\
0xF6,\
0xF6,\
0xF6,\
0xF6,\
0xF6,\
0xF6,\
0xF6,\
0xF6,\
0xF6,\
0xF7,\
0xF7,\
0xF7,\
0xF7,\
0xF7,\
0xF7,\
0xF7,\
0xF7,\
0xF7,\
0xF7,\
0xF7,\
0xF7,\
0xF7,\
0xF7,\
0xF7,\
0xF7,\
0xF7,\
0xF7,\
0xF7,\
0xF7,\
0xF7,\
0xF7,\
0xF8,\
0xF8,\
0xF8,\
0xF8,\
0xF8,\
0xF8,\
0xF8,\
0xF8,\
0xF8,\
0xF8,\
0xF8,\
0xF8,\
0xF8,\
0xF8,\
0xF8,\
0xF8,\
0xF8,\
0xF8,\
0xF8,\
0xF8,\
0xF8,\
0xF8,\
0xF8,\
0xF8,\
0xF9,\
0xF9,\
0xF9,\
0xF9,\
0xF9,\
0xF9,\
0xF9,\
0xF9,\
0xF9,\
0xF9,\
0xF9,\
0xF9,\
0xF9,\
0xF9,\
0xF9,\
0xF9,\
0xF9,\
0xF9,\
0xF9,\
0xF9,\
0xF9,\
0xF9,\
0xF9,\
0xF9,\
0xF9,\
0xF9,\
0xFA,\
0xFA,\
0xFA,\
0xFA,\
0xFA,\
0xFA,\
0xFA,\
0xFA,\
0xFA,\
0xFA,\
0xFA,\
0xFA,\
0xFA,\
0xFA,\
0xFA,\
0xFA,\
0xFA,\
0xFA,\
0xFA,\
0xFA,\
0xFA,\
0xFA,\
0xFA,\
0xFA,\
0xFA,\
0xFA,\
0xFA,\
0xFA,\
0xFA,\
0xFB,\
0xFB,\
0xFB,\
0xFB,\
0xFB,\
0xFB,\
0xFB,\
0xFB,\
0xFB,\
0xFB,\
0xFB,\
0xFB,\
0xFB,\
0xFB,\
0xFB,\
0xFB,\
0xFB,\
0xFB,\
0xFB,\
0xFB,\
0xFB,\
0xFB,\
0xFB,\
0xFB,\
0xFB,\
0xFB,\
0xFB,\
0xFB,\
0xFB,\
0xFB,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFC,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFD,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
0xFE,\
}

#define USERMATH_nSQRT_DATA	\
{0,   \
31,   \
44,   \
54,   \
63,   \
70,   \
77,   \
83,   \
89,   \
94,   \
100,   \
104,   \
109,   \
114,   \
118,   \
122,   \
126,   \
130,   \
134,   \
137,   \
141,   \
144,   \
148,   \
151,   \
154,   \
158,   \
161,   \
164,   \
167,   \
170,   \
173,   \
176,   \
178,   \
181,   \
184,   \
187,   \
189,   \
192,   \
194,   \
197,   \
200,   \
202,   \
204,   \
207,   \
209,   \
212,   \
214,   \
216,   \
219,   \
221,   \
223,   \
225,   \
228,   \
230,   \
232,   \
234,   \
236,   \
238,   \
240,   \
242,   \
244,   \
246,   \
248,   \
250,   \
252,   \
254,   \
256,   \
258,   \
260,   \
262,   \
264,   \
266,   \
268,   \
270,   \
272,   \
273,   \
275,   \
277,   \
279,   \
281,   \
282,   \
284,   \
286,   \
288,   \
289,   \
291,   \
293,   \
294,   \
296,   \
298,   \
300,   \
301,   \
303,   \
304,   \
306,   \
308,   \
309,   \
311,   \
313,   \
314,   \
316,   \
317,   \
319,   \
320,   \
322,   \
324,   \
325,   \
327,   \
328,   \
330,   \
331,   \
333,   \
334,   \
336,   \
337,   \
339,   \
340,   \
342,   \
343,   \
344,   \
346,   \
347,   \
349,   \
350,   \
352,   \
353,   \
354,   \
356,   \
357,   \
359,   \
360,   \
361,   \
363,   \
364,   \
366,   \
367,   \
368,   \
370,   \
371,   \
372,   \
374,   \
375,   \
376,   \
378,   \
379,   \
380,   \
382,   \
383,   \
384,   \
386,   \
387,   \
388,   \
389,   \
391,   \
392,   \
393,   \
394,   \
396,   \
397,   \
398,   \
400,   \
401,   \
402,   \
403,   \
404,   \
406,   \
407,   \
408,   \
409,   \
411,   \
412,   \
413,   \
414,   \
415,   \
417,   \
418,   \
419,   \
420,   \
421,   \
423,   \
424,   \
425,   \
426,   \
427,   \
428,   \
430,   \
431,   \
432,   \
433,   \
434,   \
435,   \
437,   \
438,   \
439,   \
440,   \
441,   \
442,   \
443,   \
444,   \
446,   \
447,   \
448,   \
449,   \
450,   \
451,   \
452,   \
453,   \
454,   \
456,   \
457,   \
458,   \
459,   \
460,   \
461,   \
462,   \
463,   \
464,   \
465,   \
466,   \
467,   \
469,   \
470,   \
471,   \
472,   \
473,   \
474,   \
475,   \
476,   \
477,   \
478,   \
479,   \
480,   \
481,   \
482,   \
483,   \
484,   \
485,   \
486,   \
487,   \
488,   \
489,   \
490,   \
491,   \
492,   \
493,   \
494,   \
495,   \
496,   \
497,   \
498,   \
500,   \
500,   \
501,   \
502,   \
503,   \
504,   \
505,   \
506,   \
507,   \
508,   \
509,   \
510,   \
511,   \
512,   \
513,   \
514,   \
515,   \
516,   \
517,   \
518,   \
519,   \
520,   \
521,   \
522,   \
523,   \
524,   \
525,   \
526,   \
527,   \
528,   \
529,   \
530,   \
531,   \
531,   \
532,   \
533,   \
534,   \
535,   \
536,   \
537,   \
538,   \
539,   \
540,   \
541,   \
542,   \
543,   \
544,   \
544,   \
545,   \
546,   \
547,   \
548,   \
549,   \
550,   \
551,   \
552,   \
553,   \
554,   \
554,   \
555,   \
556,   \
557,   \
558,   \
559,   \
560,   \
561,   \
562,   \
563,   \
563,   \
564,   \
565,   \
566,   \
567,   \
568,   \
569,   \
570,   \
570,   \
571,   \
572,   \
573,   \
574,   \
575,   \
576,   \
577,   \
577,   \
578,   \
579,   \
580,   \
581,   \
582,   \
583,   \
583,   \
584,   \
585,   \
586,   \
587,   \
588,   \
589,   \
589,   \
590,   \
591,   \
592,   \
593,   \
594,   \
594,   \
595,   \
596,   \
597,   \
598,   \
599,   \
600,   \
600,   \
601,   \
602,   \
603,   \
604,   \
604,   \
605,   \
606,   \
607,   \
608,   \
609,   \
609,   \
610,   \
611,   \
612,   \
613,   \
614,   \
614,   \
615,   \
616,   \
617,   \
618,   \
618,   \
619,   \
620,   \
621,   \
622,   \
622,   \
623,   \
624,   \
625,   \
626,   \
626,   \
627,   \
628,   \
629,   \
630,   \
630,   \
631,   \
632,   \
633,   \
634,   \
634,   \
635,   \
636,   \
637,   \
637,   \
638,   \
639,   \
640,   \
641,   \
641,   \
642,   \
643,   \
644,   \
644,   \
645,   \
646,   \
647,   \
648,   \
648,   \
649,   \
650,   \
651,   \
651,   \
652,   \
653,   \
654,   \
654,   \
655,   \
656,   \
657,   \
658,   \
658,   \
659,   \
660,   \
661,   \
661,   \
662,   \
663,   \
664,   \
664,   \
665,   \
666,   \
667,   \
667,   \
668,   \
669,   \
670,   \
670,   \
671,   \
672,   \
673,   \
673,   \
674,   \
675,   \
676,   \
676,   \
677,   \
678,   \
678,   \
679,   \
680,   \
681,   \
681,   \
682,   \
683,   \
684,   \
684,   \
685,   \
686,   \
687,   \
687,   \
688,   \
689,   \
689,   \
690,   \
691,   \
692,   \
692,   \
693,   \
694,   \
694,   \
695,   \
696,   \
697,   \
697,   \
698,   \
699,   \
700,   \
700,   \
701,   \
702,   \
702,   \
703,   \
704,   \
704,   \
705,   \
706,   \
707,   \
707,   \
708,   \
709,   \
709,   \
710,   \
711,   \
712,   \
712,   \
713,   \
714,   \
714,   \
715,   \
716,   \
716,   \
717,   \
718,   \
719,   \
719,   \
720,   \
721,   \
721,   \
722,   \
723,   \
723,   \
724,   \
725,   \
725,   \
726,   \
727,   \
728,   \
728,   \
729,   \
730,   \
730,   \
731,   \
732,   \
732,   \
733,   \
734,   \
734,   \
735,   \
736,   \
736,   \
737,   \
738,   \
738,   \
739,   \
740,   \
740,   \
741,   \
742,   \
742,   \
743,   \
744,   \
744,   \
745,   \
746,   \
746,   \
747,   \
748,   \
748,   \
749,   \
750,   \
750,   \
751,   \
752,   \
752,   \
753,   \
754,   \
754,   \
755,   \
756,   \
756,   \
757,   \
758,   \
758,   \
759,   \
760,   \
760,   \
761,   \
762,   \
762,   \
763,   \
764,   \
764,   \
765,   \
766,   \
766,   \
767,   \
768,   \
768,   \
769,   \
770,   \
770,   \
771,   \
772,   \
772,   \
773,   \
773,   \
774,   \
775,   \
775,   \
776,   \
777,   \
777,   \
778,   \
779,   \
779,   \
780,   \
781,   \
781,   \
782,   \
782,   \
783,   \
784,   \
784,   \
785,   \
786,   \
786,   \
787,   \
788,   \
788,   \
789,   \
789,   \
790,   \
791,   \
791,   \
792,   \
793,   \
793,   \
794,   \
794,   \
795,   \
796,   \
796,   \
797,   \
798,   \
798,   \
799,   \
800,   \
800,   \
801,   \
801,   \
802,   \
803,   \
803,   \
804,   \
804,   \
805,   \
806,   \
806,   \
807,   \
808,   \
808,   \
809,   \
809,   \
810,   \
811,   \
811,   \
812,   \
813,   \
813,   \
814,   \
814,   \
815,   \
816,   \
816,   \
817,   \
817,   \
818,   \
819,   \
819,   \
820,   \
820,   \
821,   \
822,   \
822,   \
823,   \
824,   \
824,   \
825,   \
825,   \
826,   \
827,   \
827,   \
828,   \
828,   \
829,   \
830,   \
830,   \
831,   \
831,   \
832,   \
833,   \
833,   \
834,   \
834,   \
835,   \
836,   \
836,   \
837,   \
837,   \
838,   \
839,   \
839,   \
840,   \
840,   \
841,   \
842,   \
842,   \
843,   \
843,   \
844,   \
844,   \
845,   \
846,   \
846,   \
847,   \
847,   \
848,   \
849,   \
849,   \
850,   \
850,   \
851,   \
852,   \
852,   \
853,   \
853,   \
854,   \
854,   \
855,   \
856,   \
856,   \
857,   \
857,   \
858,   \
859,   \
859,   \
860,   \
860,   \
861,   \
861,   \
862,   \
863,   \
863,   \
864,   \
864,   \
865,   \
866,   \
866,   \
867,   \
867,   \
868,   \
868,   \
869,   \
870,   \
870,   \
871,   \
871,   \
872,   \
872,   \
873,   \
874,   \
874,   \
875,   \
875,   \
876,   \
876,   \
877,   \
878,   \
878,   \
879,   \
879,   \
880,   \
880,   \
881,   \
882,   \
882,   \
883,   \
883,   \
884,   \
884,   \
885,   \
886,   \
886,   \
887,   \
887,   \
888,   \
888,   \
889,   \
889,   \
890,   \
891,   \
891,   \
892,   \
892,   \
893,   \
893,   \
894,   \
894,   \
895,   \
896,   \
896,   \
897,   \
897,   \
898,   \
898,   \
899,   \
900,   \
900,   \
901,   \
901,   \
902,   \
902,   \
903,   \
903,   \
904,   \
904,   \
905,   \
906,   \
906,   \
907,   \
907,   \
908,   \
908,   \
909,   \
909,   \
910,   \
911,   \
911,   \
912,   \
912,   \
913,   \
913,   \
914,   \
914,   \
915,   \
915,   \
916,   \
917,   \
917,   \
918,   \
918,   \
919,   \
919,   \
920,   \
920,   \
921,   \
921,   \
922,   \
923,   \
923,   \
924,   \
924,   \
925,   \
925,   \
926,   \
926,   \
927,   \
927,   \
928,   \
928,   \
929,   \
930,   \
930,   \
931,   \
931,   \
932,   \
932,   \
933,   \
933,   \
934,   \
934,   \
935,   \
935,   \
936,   \
937,   \
937,   \
938,   \
938,   \
939,   \
939,   \
940,   \
940,   \
941,   \
941,   \
942,   \
942,   \
943,   \
943,   \
944,   \
944,   \
945,   \
946,   \
946,   \
947,   \
947,   \
948,   \
948,   \
949,   \
949,   \
950,   \
950,   \
951,   \
951,   \
952,   \
952,   \
953,   \
953,   \
954,   \
954,   \
955,   \
956,   \
956,   \
957,   \
957,   \
958,   \
958,   \
959,   \
959,   \
960,   \
960,   \
961,   \
961,   \
962,   \
962,   \
963,   \
963,   \
964,   \
964,   \
965,   \
965,   \
966,   \
966,   \
967,   \
967,   \
968,   \
969,   \
969,   \
970,   \
970,   \
971,   \
971,   \
972,   \
972,   \
973,   \
973,   \
974,   \
974,   \
975,   \
975,   \
976,   \
976,   \
977,   \
977,   \
978,   \
978,   \
979,   \
979,   \
980,   \
980,   \
981,   \
981,   \
982,   \
982,   \
983,   \
983,   \
984,   \
984,   \
985,   \
985,   \
986,   \
986,   \
987,   \
987,   \
988,   \
988,   \
989,   \
989,   \
990,   \
990,   \
991,   \
991,   \
992,   \
992,   \
993,   \
993,   \
994,   \
994,   \
995,   \
995,   \
996,   \
996,   \
997,   \
997,   \
998,   \
998,   \
999}

#define CRC16_DATA \
{\
	0x0000, 0x1189, 0x2312, 0x329B, 0x4624, 0x57AD, 0x6536, 0x74BF,\
	0x8C48, 0x9DC1, 0xAF5A, 0xBED3, 0xCA6C, 0xDBE5, 0xE97E, 0xF8F7,\
	0x0919, 0x1890, 0x2A0B, 0x3B82, 0x4F3D, 0x5EB4, 0x6C2F, 0x7DA6,\
	0x8551, 0x94D8, 0xA643, 0xB7CA, 0xC375, 0xD2FC, 0xE067, 0xF1EE,\
	0x1232, 0x03BB, 0x3120, 0x20A9, 0x5416, 0x459F, 0x7704, 0x668D,\
	0x9E7A, 0x8FF3, 0xBD68, 0xACE1, 0xD85E, 0xC9D7, 0xFB4C, 0xEAC5,\
	0x1B2B, 0x0AA2, 0x3839, 0x29B0, 0x5D0F, 0x4C86, 0x7E1D, 0x6F94,\
	0x9763, 0x86EA, 0xB471, 0xA5F8, 0xD147, 0xC0CE, 0xF255, 0xE3DC,\
	0x2464, 0x35ED, 0x0776, 0x16FF, 0x6240, 0x73C9, 0x4152, 0x50DB,\
	0xA82C, 0xB9A5, 0x8B3E, 0x9AB7, 0xEE08, 0xFF81, 0xCD1A, 0xDC93,\
	0x2D7D, 0x3CF4, 0x0E6F, 0x1FE6, 0x6B59, 0x7AD0, 0x484B, 0x59C2,\
	0xA135, 0xB0BC, 0x8227, 0x93AE, 0xE711, 0xF698, 0xC403, 0xD58A,\
	0x3656, 0x27DF, 0x1544, 0x04CD, 0x7072, 0x61FB, 0x5360, 0x42E9,\
	0xBA1E, 0xAB97, 0x990C, 0x8885, 0xFC3A, 0xEDB3, 0xDF28, 0xCEA1,\
	0x3F4F, 0x2EC6, 0x1C5D, 0x0DD4, 0x796B, 0x68E2, 0x5A79, 0x4BF0,\
	0xB307, 0xA28E, 0x9015, 0x819C, 0xF523, 0xE4AA, 0xD631, 0xC7B8,\
	0x48C8, 0x5941, 0x6BDA, 0x7A53, 0x0EEC, 0x1F65, 0x2DFE, 0x3C77,\
	0xC480, 0xD509, 0xE792, 0xF61B, 0x82A4, 0x932D, 0xA1B6, 0xB03F,\
	0x41D1, 0x5058, 0x62C3, 0x734A, 0x07F5, 0x167C, 0x24E7, 0x356E,\
	0xCD99, 0xDC10, 0xEE8B, 0xFF02, 0x8BBD, 0x9A34, 0xA8AF, 0xB926,\
	0x5AFA, 0x4B73, 0x79E8, 0x6861, 0x1CDE, 0x0D57, 0x3FCC, 0x2E45,\
	0xD6B2, 0xC73B, 0xF5A0, 0xE429, 0x9096, 0x811F, 0xB384, 0xA20D,\
	0x53E3, 0x426A, 0x70F1, 0x6178, 0x15C7, 0x044E, 0x36D5, 0x275C,\
	0xDFAB, 0xCE22, 0xFCB9, 0xED30, 0x998F, 0x8806, 0xBA9D, 0xAB14,\
	0x6CAC, 0x7D25, 0x4FBE, 0x5E37, 0x2A88, 0x3B01, 0x099A, 0x1813,\
	0xE0E4, 0xF16D, 0xC3F6, 0xD27F, 0xA6C0, 0xB749, 0x85D2, 0x945B,\
	0x65B5, 0x743C, 0x46A7, 0x572E, 0x2391, 0x3218, 0x0083, 0x110A,\
	0xE9FD, 0xF874, 0xCAEF, 0xDB66, 0xAFD9, 0xBE50, 0x8CCB, 0x9D42,\
	0x7E9E, 0x6F17, 0x5D8C, 0x4C05, 0x38BA, 0x2933, 0x1BA8, 0x0A21,\
	0xF2D6, 0xE35F, 0xD1C4, 0xC04D, 0xB4F2, 0xA57B, 0x97E0, 0x8669,\
	0x7787, 0x660E, 0x5495, 0x451C, 0x31A3, 0x202A, 0x12B1, 0x0338,\
	0xFBCF, 0xEA46, 0xD8DD, 0xC954, 0xBDEB, 0xAC62, 0x9EF9, 0x8F70\
}


/* GLOBAL FUNCTION DECLARATIONS ***********************************************/
uint8 USERMATH_u8GetFilterFromTimeConstant(uint32, uint32);
uint16 USERMATH_u16SinglePoleLowPassFilter16(uint16 u16NewVal, uint8 u8FilterWeight, puint32 pu32FilterVal);/*CR1_1*/
uint32 USERMATH_u32SinglePoleLowPassFilter32(uint32 u32NewVal, uint8 u8FilterWeight, puint32 pu32FilterVal);/*CR1_1*/
uint32 USERMATH_u32DiscardAndAverage32(puint32 pu32Samples, uint32 u32SampleCount, uint32 u32DiscardCount);/*CR1_1*/
sint32 USERMATH_s32DiscardAndAverage32(psint32 ps32Samples, uint32 u32SampleCount, uint32 u32DiscardCount);/*CR1_1*/
uint32 USERMATH_u32GetSquareRoot(uint32 u32Fraction);
uint16 USERMATH_U16GetCRC(uint16 CRCSeed, puint8 pu8Data, uint32 u32Len);

#endif // UNITS_H

